#!/usr/bin/python3
import gitlab, os, re, argparse, sys
from git import Repo

parser = argparse.ArgumentParser()
parser.add_argument('-f', '--file', type=str, default='.gitlab-ci-local-variables.yml')
parser.add_argument('-e', '--envs', nargs='+', help='Choose environment scope (Try --list before).', type=str)
parser.add_argument('-g', '--get', help='Fetch variables from gitlab.', action='store_true')
parser.add_argument('-p', '--push', help='Push variables to gitlab.', action='store_true')
parser.add_argument('--force', help='Force push variables to gitlab.', action='store_true')
parser.add_argument('-l', '--list', help='List environment scopes.', action='store_true')
# parser.add_argument('-j', '--json', help='Gen JSON varfile.', action='store_true')

args = parser.parse_args()
repo = Repo(os.getcwd())
protocol, domain, path = re.split('@|:', repo.remote().url.split('.git')[0])
url = 'https://' + domain

'''
Create Gitlab instanse.
"keep_base_url=True" needs to resolve warning:
"UserWarning: The base URL in the server response differs from the user-provided base URL (https://gitlab.example.com -> http://gitlab.example.com)."
'''
gl = gitlab.Gitlab(url, private_token=os.environ['GITLAB_TOKEN'],  keep_base_url=True)

project = gl.projects.get(path, lazy=True)                          # Create project's object
project_variables = project.variables.list(get_all=True)            # Get variables

vars_dict = {}
parse_dict = {}


def gen_vars_dict():
    for variable in project_variables:                                  # Create dict of environment scopes
        vars_dict.update({variable.environment_scope:{}})

    for env_scope in vars_dict:                                         # Filling environment scopes by key:value
        for variable in project_variables:
            if variable.environment_scope == env_scope:
                vars_dict[env_scope].update({variable.key:variable.value})

def gen_varfile_json():
    with open(args.file, 'w') as f:
        for env in vars_dict:
            f.write('###  Environment scope: "{env_scope}" ###\n'.format(env_scope = env))
            tmp_lines = []
            for var in vars_dict[env]:
                f.write('# {key}: "{value}"\n'.format(key = var, value = vars_dict[env][var]))
            f.write(''.join(tmp_lines))
            f.write('\n')
    print('Variables are written to the file: "{file}".'.format(file = args.file))

def print_stdout_json():
    for env in vars_dict:
        print('###  Environment scope: "{env_scope}" ###'.format(env_scope = env))
        tmp_lines = []
        for var in vars_dict[env]:
            re_expression = re.findall(" \${[0-9A-Z_]*}", vars_dict[env][var])
            if len(re_expression) > 0:
                tmp_lines.append('# {key}: "{value}"'.format(key = var, value = vars_dict[env][var]))
                continue
            print('# {key}: "{value}"'.format(key = var, value = vars_dict[env][var]))
        print(''.join(tmp_lines))
        print('')

def print_envs():
    print('List of environment scopes: ')
    for env in vars_dict:
        print(env)

def select_envs():
    del_list = []
    for env in vars_dict:
        if env not in args.envs:
            del_list.append(env)
    for env in del_list:
        del vars_dict[env]
        print('Env "%s" deleted from output.' % env)
    print('')

def parse_varfile_json():
    var_re = ''
    del_list = []
    with open(args.file, 'r') as f:
        for line in f.readlines():
            env_re = re.findall('Environment scope: "(.*)"', line)
            var_re = re.findall('([a-zA-Z_].*): "(.*)"', line)
            if 'Environment scope:' in line:
                parse_dict.update({env_re[0]:{}})
                env_scope = env_re[0]
            else:
                vars = dict((key, value) for key, value in var_re)
                parse_dict[env_scope].update(vars)
    if not args.force:                                  # If env isn't exist in "parse_dict" (vars from file)
        for env in vars_dict:                             # remove it from "vars_dict" (vars from gitlab)
            if env not in parse_dict:                     # for take effect (create, update, delete)
                del_list.append(env)                      # only to variables existing in  "parse_dict"
        for env in del_list:
            del vars_dict[env]

def gen_push_list():
    push_list = []
    if vars_dict == parse_dict:
        print('No changes found.')
    else:
        for env_scope in parse_dict:
            for variable in parse_dict[env_scope]:
                if (env_scope not in vars_dict) or (variable not in vars_dict[env_scope]):
                    push_list.append({'key': variable,
                                      'value': parse_dict[env_scope][variable],
                                      'environment_scope': env_scope,
                                      'action':'create'})

                elif (variable in vars_dict[env_scope]) and (vars_dict[env_scope][variable] != parse_dict[env_scope][variable]):
                    push_list.append({'key': variable,
                                      'value': parse_dict[env_scope][variable],
                                      'environment_scope': env_scope,
                                      'action':'update'})

        for env_scope in vars_dict:
            for variable in vars_dict[env_scope]:
                if (env_scope not in parse_dict):
                    push_list.append({'key': variable,
                                      'environment_scope': env_scope,
                                      'action':'delete'})

                elif (variable in vars_dict[env_scope]) and (variable not in parse_dict[env_scope]):
                    push_list.append({'key': variable,
                                      'environment_scope': env_scope,
                                      'action':'delete'})
    return push_list

def push_vars(push_list):
    for i in push_list:
        if i['action'] == 'create':
            project.variables.create({'key': i['key'], 'value': i['value'], 'environment_scope': i['environment_scope']})
            print('Created: %s' % i)
        elif i['action'] == 'update':
            project.variables.update(i['key'], {'value': i['value']}, filter={'environment_scope': i['environment_scope']})
            print('Updated: %s' % i)
        elif i['action'] == 'delete':
            project.variables.delete(i['key'], filter={'environment_scope': i['environment_scope']})
            print('Deleted: %s' % i)

def main():

  gen_vars_dict()

  if args.envs and args.get:
      select_envs()
      gen_varfile_json()
  elif args.get:
      gen_varfile_json()
  elif args.push:
      parse_varfile_json()
      push_list = gen_push_list()
      push_vars(push_list)
  elif not len(sys.argv) > 1:
      print_stdout_json()
  elif args.list:
      print_envs()
  elif args.envs:
      select_envs()
      print_stdout_json()

if __name__ == '__main__':
    main()
